"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const fs_1 = __importDefault(require("fs"));
const range_1 = __importDefault(require("lodash/range"));
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const CLUSTER = process.env.CLUSTER_OVERRIDE || 'mainnet-beta';
const CLUSTER_URL = process.env.CLUSTER_URL_OVERRIDE || process.env.MB_CLUSTER_URL;
const USER_KEYPAIR = process.env.USER_KEYPAIR_OVERRIDE || process.env.MB_PAYER_KEYPAIR;
const GROUP_PK = process.env.GROUP_PK || '78b8f4cGCwmZ9ysPFMWLaLTkkaYnUjwMJYStWe5RTSSX';
const MARKET_INDEX = Number(process.env.MARKET_INDEX);
async function forceCloseSerum3Market() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection(CLUSTER_URL, options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(process.env.KEYPAIR || fs_1.default.readFileSync(USER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, CLUSTER, constants_1.MANGO_V4_ID[CLUSTER], {
        idsSource: 'get-program-accounts',
    });
    const group = await client.getGroup(new web3_js_1.PublicKey(GROUP_PK));
    const serum3Market = group.serum3MarketsMapByMarketIndex.get(MARKET_INDEX);
    if (!serum3Market.reduceOnly) {
        throw new Error(`Unexpected reduce only state ${serum3Market.reduceOnly}`);
    }
    if (!serum3Market.forceClose) {
        throw new Error(`Unexpected force close state ${serum3Market.forceClose}`);
    }
    // Get all mango accounts who have a serum oo account for the given market
    const mangoAccounts = (await client.getAllMangoAccounts(group, true)).filter((a) => a.serum3OosMapByMarketIndex.get(MARKET_INDEX) !== undefined);
    for (let a of mangoAccounts) {
        // Cancel all orders and confirm that all have been cancelled
        for (const _ of (0, range_1.default)(0, 10)) {
            console.log(a.getSerum3OoAccount(MARKET_INDEX).freeSlotBits.zeroBits());
            const sig = await client.serum3LiqForceCancelOrders(group, a, serum3Market.serumMarketExternal, 10);
            console.log(` serum3LiqForceCancelOrders for ${a.publicKey}, sig https://explorer.solana.com/tx/${sig}?cluster=${CLUSTER == 'devnet' ? 'devnet' : ''}`);
            a = await a.reload(client);
            if (a.getSerum3OoAccount(MARKET_INDEX).freeSlotBits.zeroBits() === 0) {
                break;
            }
        }
    }
}
forceCloseSerum3Market();
