"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProposal = exports.MANGO_GOVERNANCE_PROGRAM = exports.MANGO_REALM_PK = exports.MANGO_MINT = void 0;
const spl_governance_1 = require("@solana/spl-governance");
const web3_js_1 = require("@solana/web3.js");
const chunk_1 = __importDefault(require("lodash/chunk"));
const updateVoteWeightRecord_1 = require("./updateVoteWeightRecord");
exports.MANGO_MINT = 'MangoCzJ36AjZyKwVj3VnYU4GTonjfVEnJmvvWaxLac';
exports.MANGO_REALM_PK = new web3_js_1.PublicKey('DPiH3H3c7t47BMxqTxLsuPQpEC6Kne8GA9VXbxpnZxFE');
exports.MANGO_GOVERNANCE_PROGRAM = new web3_js_1.PublicKey('GqTPL6qRf5aUuqscLh8Rg2HTxPUXfhhAXDptTLhp1t2J');
const createProposal = async (connection, wallet, governance, tokenOwnerRecord, name, descriptionLink, proposalIndex, proposalInstructions, client, signOff) => {
    const instructions = [];
    const walletPk = wallet.publicKey;
    const governanceAuthority = walletPk;
    const signatory = walletPk;
    const payer = walletPk;
    // Changed this because it is misbehaving on my local validator setup.
    const programVersion = await (0, spl_governance_1.getGovernanceProgramVersion)(connection, exports.MANGO_GOVERNANCE_PROGRAM);
    // V2 Approve/Deny configuration
    const voteType = spl_governance_1.VoteType.SINGLE_CHOICE;
    const options = ['Approve'];
    const useDenyOption = true;
    const { updateVoterWeightRecordIx, voterWeightPk } = await (0, updateVoteWeightRecord_1.updateVoterWeightRecord)(client, tokenOwnerRecord.account.governingTokenOwner);
    instructions.push(updateVoterWeightRecordIx);
    const proposalAddress = await (0, spl_governance_1.withCreateProposal)(instructions, exports.MANGO_GOVERNANCE_PROGRAM, programVersion, exports.MANGO_REALM_PK, governance, tokenOwnerRecord.pubkey, name, descriptionLink, new web3_js_1.PublicKey(exports.MANGO_MINT), governanceAuthority, proposalIndex, voteType, options, useDenyOption, payer, voterWeightPk);
    await (0, spl_governance_1.withAddSignatory)(instructions, exports.MANGO_GOVERNANCE_PROGRAM, programVersion, proposalAddress, tokenOwnerRecord.pubkey, governanceAuthority, signatory, payer);
    const insertInstructions = [];
    for (const i in proposalInstructions) {
        const instruction = (0, spl_governance_1.getInstructionDataFromBase64)((0, spl_governance_1.serializeInstructionToBase64)(proposalInstructions[i]));
        await (0, spl_governance_1.withInsertTransaction)(insertInstructions, exports.MANGO_GOVERNANCE_PROGRAM, programVersion, governance, proposalAddress, tokenOwnerRecord.pubkey, governanceAuthority, Number(i), 0, 0, [instruction], payer);
    }
    if (signOff) {
        const signatoryRecordAddress = await (0, spl_governance_1.getSignatoryRecordAddress)(exports.MANGO_GOVERNANCE_PROGRAM, proposalAddress, signatory);
        (0, spl_governance_1.withSignOffProposal)(insertInstructions, // SingOff proposal needs to be executed after inserting instructions hence we add it to insertInstructions
        exports.MANGO_GOVERNANCE_PROGRAM, programVersion, exports.MANGO_REALM_PK, governance, proposalAddress, signatory, signatoryRecordAddress, undefined);
    }
    const txChunks = (0, chunk_1.default)([...instructions, ...insertInstructions], 2);
    const transactions = [];
    const latestBlockhash = await connection.getLatestBlockhash('confirmed');
    for (const chunk of txChunks) {
        const tx = new web3_js_1.Transaction();
        tx.add(...chunk);
        tx.lastValidBlockHeight = latestBlockhash.lastValidBlockHeight;
        tx.recentBlockhash = latestBlockhash.blockhash;
        tx.feePayer = payer;
        transactions.push(tx);
    }
    const signedTransactions = await wallet.signAllTransactions(transactions);
    for (const tx of signedTransactions) {
        const rawTransaction = tx.serialize();
        const address = await connection.sendRawTransaction(rawTransaction, {
            skipPreflight: true,
        });
        await connection.confirmTransaction({
            blockhash: latestBlockhash.blockhash,
            lastValidBlockHeight: latestBlockhash.lastValidBlockHeight,
            signature: address,
        });
    }
    return proposalAddress;
};
exports.createProposal = createProposal;
